import fs from 'fs/promises';
import path from 'path';
import { Task, Step } from '../db/types';
import { steps } from '../db';

export class TestPrinter {
  private readonly outputDir: string;

  constructor() {
    this.outputDir = path.join(process.cwd(), 'test-output');
    this.ensureOutputDir();
  }

  private async ensureOutputDir() {
    try {
      await fs.mkdir(this.outputDir, { recursive: true });
    } catch (error) {
      console.error('Failed to create output directory:', error);
    }
  }

  async getTaskSteps(task: Task): Promise<Step[]> {
    return await steps().where('task_id', task.id).orderBy('order').select('*');
  }

  async printTask(task: Task, taskSteps: Step[]): Promise<void> {
    const timestamp = new Date().toISOString().replace(/[:.]/g, '-');
    const filename = path.join(this.outputDir, `task-${task.id}-${timestamp}.txt`);

    const content = [
      `Task: ${task.title}`,
      '='.repeat(40),
      '',
      ...taskSteps.map((step, index) => [
        `Step ${index + 1}: ${step.title}`,
        '-'.repeat(40),
        step.instructions,
        '',
      ]).flat(),
    ].join('\n');

    await fs.writeFile(filename, content);
  }

  async printStep(step: Step): Promise<void> {
    const timestamp = new Date().toISOString().replace(/[:.]/g, '-');
    const filename = path.join(this.outputDir, `step-${step.id}-${timestamp}.txt`);

    const content = [
      `Step: ${step.title}`,
      '='.repeat(40),
      '',
      step.instructions,
      '',
    ].join('\n');

    await fs.writeFile(filename, content);
  }
}